@@ -38,8 +38,10 @@
 #ifndef _RADEON_H_
 #define _RADEON_H_
 
+#include <stdlib.h>		/* For abs() */
+#include <unistd.h>		/* For usleep() */
+
 #include "xf86str.h"
-#include "xf86_ansic.h"
 #include "compiler.h"
 #include "xf86fbman.h"
 
@@ -105,7 +107,6 @@ typedef struct _region {
 #define RADEON_DEBUG            1 /* Turn off debugging output               */
 #define RADEON_IDLE_RETRY      16 /* Fall out of idle loops after this count */
 #define RADEON_TIMEOUT    2000000 /* Fall out of wait loops after this count */
-#define RADEON_MMIOSIZE   0x80000
 
 /* Buffer are aligned on 4096 byte boundaries */
 #define RADEON_BUFFER_ALIGN 0x00000fff
@@ -347,6 +348,7 @@ typedef struct {
 
     CARD32            MemCntl;
     CARD32            BusCntl;
+    unsigned long     MMIOSize;         /* MMIO region physical address      */
     unsigned long     FbMapSize;        /* Size of frame buffer, in bytes    */
     unsigned long     FbSecureSize;     /* Size of secured fb area at end of
                                            framebuffer */
@@ -409,7 +411,7 @@ typedef struct {
     Bool              PaletteSavedOnVT; /* Palette saved on last VT switch   */
 
 #ifdef USE_EXA
-    ExaDriverRec      exa;
+    ExaDriverPtr      exa;
     int               engineMode;
 #define EXA_ENGINEMODE_UNKNOWN 0
 #define EXA_ENGINEMODE_2D      1
@@ -539,20 +541,20 @@ typedef struct {
     drm_handle_t         ringHandle;       /* Handle from drmAddMap */
     drmSize           ringMapSize;      /* Size of map */
     int               ringSize;         /* Size of ring (in MB) */
-    unsigned char     *ring;            /* Map */
+    drmAddress        ring;             /* Map */
     int               ringSizeLog2QW;
 
     unsigned long     ringReadOffset;   /* Offset into GART space */
     drm_handle_t         ringReadPtrHandle; /* Handle from drmAddMap */
     drmSize           ringReadMapSize;  /* Size of map */
-    unsigned char     *ringReadPtr;     /* Map */
+    drmAddress        ringReadPtr;      /* Map */
 
 				/* CP vertex/indirect buffer data */
     unsigned long     bufStart;         /* Offset into GART space */
     drm_handle_t         bufHandle;        /* Handle from drmAddMap */
     drmSize           bufMapSize;       /* Size of map */
     int               bufSize;          /* Size of buffers (in MB) */
-    unsigned char     *buf;             /* Map */
+    drmAddress        buf;              /* Map */
     int               bufNumBufs;       /* Number of buffers */
     drmBufMapPtr      buffers;          /* Buffer map */
 
@@ -561,7 +563,7 @@ typedef struct {
     drm_handle_t         gartTexHandle;     /* Handle from drmAddMap */
     drmSize           gartTexMapSize;    /* Size of map */
     int               gartTexSize;       /* Size of GART tex space (in MB) */
-    unsigned char     *gartTex;          /* Map */
+    drmAddress        gartTex;           /* Map */
     int               log2GARTTexGran;
 
 				/* CP accleration */
@@ -709,6 +711,11 @@ typedef struct {
     Bool                NonRect, HaveNonRect, HaveOffsRegions, MouseRestrictions;
     region              NonRectDead, OffDead1, OffDead2;
 
+    int			constantDPI; /* -1 = auto, 0 = off, 1 = on */
+    int			RADEONDPIVX, RADEONDPIVY;
+    RADEONScrn2Rel	MergedDPISRel;
+    int			RADEONMergedDPIVX, RADEONMergedDPIVY, RADEONMergedDPIRot;
+
     /* special handlings for DELL triple-head server */
     Bool		IsDellServer; 
 
@@ -795,9 +802,12 @@ extern void        RADEONCPFlushIndirect(ScrnInfoPtr pScrn, int discard);
 extern void        RADEONCPReleaseIndirect(ScrnInfoPtr pScrn);
 extern int         RADEONCPStop(ScrnInfoPtr pScrn,  RADEONInfoPtr info);
 
-extern CARD8*      RADEONHostDataBlit(ScrnInfoPtr pScrn, unsigned int bpp,
-				      unsigned int w, CARD32 dstPitch,
-				      CARD32 *bufPitch, CARD8 **dst,
+extern void        RADEONHostDataParams(ScrnInfoPtr pScrn, CARD8 *dst,
+					CARD32 pitch, int cpp,
+					CARD32 *dstPitchOffset, int *x, int *y);
+extern CARD8*      RADEONHostDataBlit(ScrnInfoPtr pScrn, unsigned int cpp,
+				      unsigned int w, CARD32 dstPitchOff,
+				      CARD32 *bufPitch, int x, int *y,
 				      unsigned int *h, unsigned int *hpass);
 extern void        RADEONHostDataBlitCopyPass(ScrnInfoPtr pScrn,
 					      unsigned int bpp,
@@ -882,7 +892,7 @@ do {									\
 #define BEGIN_RING(n) do {						\
     if (RADEON_VERBOSE) {						\
 	xf86DrvMsg(pScrn->scrnIndex, X_INFO,				\
-		   "BEGIN_RING(%d) in %s\n", n, __FUNCTION__);		\
+		   "BEGIN_RING(%d) in %s\n", (unsigned int)n, __FUNCTION__);\
     }									\
     if (++info->dma_begin_count != 1) {					\
 	xf86DrvMsg(pScrn->scrnIndex, X_ERROR,				\
