@@ -129,7 +129,7 @@ static Bool RADEONInitVisualConfigs(ScreenPtr pScreen)
 	}
 
 	i = 0;
-	for (db = 0; db <= use_db; db++) {
+	for (db = use_db; db >= 0; db--) {
 	  for (accum = 0; accum <= RADEON_USE_ACCUM; accum++) {
 	    for (stencil = 0; stencil <= RADEON_USE_STENCIL; stencil++) {
 		pRADEONConfigPtrs[i] = &pRADEONConfigs[i];
@@ -212,7 +212,7 @@ static Bool RADEONInitVisualConfigs(ScreenPtr pScreen)
 	}
 
 	i = 0;
-	for (db = 0; db <= use_db; db++) {
+	for (db = use_db; db >= 0; db--) {
 	  for (accum = 0; accum <= RADEON_USE_ACCUM; accum++) {
 	    for (stencil = 0; stencil <= RADEON_USE_STENCIL; stencil++) {
 		pRADEONConfigPtrs[i] = &pRADEONConfigs[i];
@@ -728,8 +728,9 @@ static Bool RADEONSetAgpMode(RADEONInfoPtr info, ScreenPtr pScreen)
     mode &= ~RADEON_AGP_MODE_MASK;
     if ((mode & RADEON_AGPv3_MODE) &&
 	(INREG(RADEON_AGP_STATUS) & RADEON_AGPv3_MODE)) {
+	/* only set one mode bit for AGPv3 */
 	switch (info->agpMode) {
-	case 8:          mode |= RADEON_AGPv3_8X_MODE;
+	case 8:          mode |= RADEON_AGPv3_8X_MODE; break;
 	case 4: default: mode |= RADEON_AGPv3_4X_MODE;
 	}
 	/*TODO: need to take care of other bits valid for v3 mode
@@ -815,7 +816,7 @@ static Bool RADEONDRIAgpInit(RADEONInfoPtr info, ScreenPtr pScreen)
 	return FALSE;
     }
     xf86DrvMsg(pScreen->myNum, X_INFO,
-	       "[agp] %d kB allocated with handle 0x%08lx\n",
+	       "[agp] %d kB allocated with handle 0x%08x\n",
 	       info->gartSize*1024, info->agpMemHandle);
 
     if (drmAgpBind(info->drmFD,
@@ -836,7 +837,7 @@ static Bool RADEONDRIAgpInit(RADEONInfoPtr info, ScreenPtr pScreen)
 	       "[agp] ring handle = 0x%08x\n", info->ringHandle);
 
     if (drmMap(info->drmFD, info->ringHandle, info->ringMapSize,
-	       (drmAddressPtr)&info->ring) < 0) {
+	       &info->ring) < 0) {
 	xf86DrvMsg(pScreen->myNum, X_ERROR, "[agp] Could not map ring\n");
 	return FALSE;
     }
@@ -855,7 +856,7 @@ static Bool RADEONDRIAgpInit(RADEONInfoPtr info, ScreenPtr pScreen)
 	       info->ringReadPtrHandle);
 
     if (drmMap(info->drmFD, info->ringReadPtrHandle, info->ringReadMapSize,
-	       (drmAddressPtr)&info->ringReadPtr) < 0) {
+	       &info->ringReadPtr) < 0) {
 	xf86DrvMsg(pScreen->myNum, X_ERROR,
 		   "[agp] Could not map ring read ptr\n");
 	return FALSE;
@@ -875,7 +876,7 @@ static Bool RADEONDRIAgpInit(RADEONInfoPtr info, ScreenPtr pScreen)
 	       info->bufHandle);
 
     if (drmMap(info->drmFD, info->bufHandle, info->bufMapSize,
-	       (drmAddressPtr)&info->buf) < 0) {
+	       &info->buf) < 0) {
 	xf86DrvMsg(pScreen->myNum, X_ERROR,
 		   "[agp] Could not map vertex/indirect buffers\n");
 	return FALSE;
@@ -895,7 +896,7 @@ static Bool RADEONDRIAgpInit(RADEONInfoPtr info, ScreenPtr pScreen)
 	       info->gartTexHandle);
 
     if (drmMap(info->drmFD, info->gartTexHandle, info->gartTexMapSize,
-	       (drmAddressPtr)&info->gartTex) < 0) {
+	       &info->gartTex) < 0) {
 	xf86DrvMsg(pScreen->myNum, X_ERROR,
 		   "[agp] Could not map GART texture map\n");
 	return FALSE;
@@ -924,7 +925,7 @@ static Bool RADEONDRIPciInit(RADEONInfoPtr info, ScreenPtr pScreen)
 	return FALSE;
     }
     xf86DrvMsg(pScreen->myNum, X_INFO,
-	       "[pci] %d kB allocated with handle 0x%08lx\n",
+	       "[pci] %d kB allocated with handle 0x%08x\n",
 	       info->gartSize*1024, info->pciMemHandle);
 
     RADEONDRIInitGARTValues(info);
@@ -939,7 +940,7 @@ static Bool RADEONDRIPciInit(RADEONInfoPtr info, ScreenPtr pScreen)
 	       "[pci] ring handle = 0x%08x\n", info->ringHandle);
 
     if (drmMap(info->drmFD, info->ringHandle, info->ringMapSize,
-	       (drmAddressPtr)&info->ring) < 0) {
+	       &info->ring) < 0) {
 	xf86DrvMsg(pScreen->myNum, X_ERROR, "[pci] Could not map ring\n");
 	return FALSE;
     }
@@ -961,7 +962,7 @@ static Bool RADEONDRIPciInit(RADEONInfoPtr info, ScreenPtr pScreen)
 	       info->ringReadPtrHandle);
 
     if (drmMap(info->drmFD, info->ringReadPtrHandle, info->ringReadMapSize,
-	       (drmAddressPtr)&info->ringReadPtr) < 0) {
+	       &info->ringReadPtr) < 0) {
 	xf86DrvMsg(pScreen->myNum, X_ERROR,
 		   "[pci] Could not map ring read ptr\n");
 	return FALSE;
@@ -984,7 +985,7 @@ static Bool RADEONDRIPciInit(RADEONInfoPtr info, ScreenPtr pScreen)
 	       info->bufHandle);
 
     if (drmMap(info->drmFD, info->bufHandle, info->bufMapSize,
-	       (drmAddressPtr)&info->buf) < 0) {
+	       &info->buf) < 0) {
 	xf86DrvMsg(pScreen->myNum, X_ERROR,
 		   "[pci] Could not map vertex/indirect buffers\n");
 	return FALSE;
@@ -1007,7 +1008,7 @@ static Bool RADEONDRIPciInit(RADEONInfoPtr info, ScreenPtr pScreen)
 	       info->gartTexHandle);
 
     if (drmMap(info->drmFD, info->gartTexHandle, info->gartTexMapSize,
-	       (drmAddressPtr)&info->gartTex) < 0) {
+	       &info->gartTex) < 0) {
 	xf86DrvMsg(pScreen->myNum, X_ERROR,
 		   "[pci] Could not map GART texture map\n");
 	return FALSE;
@@ -1025,13 +1026,13 @@ static Bool RADEONDRIPciInit(RADEONInfoPtr info, ScreenPtr pScreen)
 static Bool RADEONDRIMapInit(RADEONInfoPtr info, ScreenPtr pScreen)
 {
 				/* Map registers */
-    info->registerSize = RADEON_MMIOSIZE;
+    info->registerSize = info->MMIOSize;
     if (drmAddMap(info->drmFD, info->MMIOAddr, info->registerSize,
 		  DRM_REGISTERS, DRM_READ_ONLY, &info->registerHandle) < 0) {
 	return FALSE;
     }
     xf86DrvMsg(pScreen->myNum, X_INFO,
-	       "[drm] register handle = 0x%08lx\n", info->registerHandle);
+	       "[drm] register handle = 0x%08x\n", info->registerHandle);
 
     return TRUE;
 }
@@ -1094,24 +1095,18 @@ static void RADEONDRIGartHeapInit(RADEONInfoPtr info, ScreenPtr pScreen)
     drmRadeonMemInitHeap drmHeap;
 
     /* Start up the simple memory manager for GART space */
-    if (info->pKernelDRMVersion->version_minor >= 6) {
-	drmHeap.region = RADEON_MEM_REGION_GART;
-	drmHeap.start  = 0;
-	drmHeap.size   = info->gartTexMapSize;
+    drmHeap.region = RADEON_MEM_REGION_GART;
+    drmHeap.start  = 0;
+    drmHeap.size   = info->gartTexMapSize;
 
-	if (drmCommandWrite(info->drmFD, DRM_RADEON_INIT_HEAP,
-			    &drmHeap, sizeof(drmHeap))) {
-	    xf86DrvMsg(pScreen->myNum, X_ERROR,
-		       "[drm] Failed to initialize GART heap manager\n");
-	} else {
-	    xf86DrvMsg(pScreen->myNum, X_INFO,
-		       "[drm] Initialized kernel GART heap manager, %d\n",
-		       info->gartTexMapSize);
-	}
+    if (drmCommandWrite(info->drmFD, DRM_RADEON_INIT_HEAP,
+			&drmHeap, sizeof(drmHeap))) {
+	xf86DrvMsg(pScreen->myNum, X_ERROR,
+		   "[drm] Failed to initialize GART heap manager\n");
     } else {
 	xf86DrvMsg(pScreen->myNum, X_INFO,
-		   "[drm] Kernel module too old (1.%d) for GART heap manager\n",
-		   info->pKernelDRMVersion->version_minor);
+		   "[drm] Initialized kernel GART heap manager, %d\n",
+		   info->gartTexMapSize);
     }
 }
 
@@ -1202,7 +1197,6 @@ Bool RADEONDRIGetVersion(ScrnInfoPtr pScrn)
     int            major, minor, patch, fd;
     int		   req_minor, req_patch;
     char           *busId;
-    drmVersionPtr  libVersion;
 
     /* Check that the GLX, DRI, and DRM modules have been loaded by testing
      * for known symbols in each module.
@@ -1296,11 +1290,8 @@ Bool RADEONDRIGetVersion(ScrnInfoPtr pScrn)
     } else if (info->IsIGP) {
         req_minor = 10;
 	req_patch = 0;
-    } else if (info->ChipFamily >= CHIP_FAMILY_R200) {
-	req_minor = 5;
-	req_patch = 0;
-    } else {
-	req_minor = 3;
+    } else { /* Many problems have been reported with 1.7 in the 2.4 kernel */
+	req_minor = 8;
 	req_patch = 0;
     }
 
@@ -1340,7 +1331,6 @@ Bool RADEONDRIScreenInit(ScreenPtr pScreen)
     RADEONInfoPtr  info    = RADEONPTR(pScrn);
     DRIInfoPtr     pDRIInfo;
     RADEONDRIPtr   pRADEONDRI;
-    drmVersionPtr  version;
 
     info->DRICloseScreen = NULL;
 
@@ -1765,6 +1755,11 @@ void RADEONDRICloseScreen(ScreenPtr pScreen)
 	info->pciMemHandle = 0;
     }
 
+    if (info->pciGartBackup) {
+	xfree(info->pciGartBackup);
+	info->pciGartBackup = NULL;
+    }
+
     /* De-allocate all DRI resources */
     DRICloseScreen(pScreen);
 
@@ -1985,7 +1980,7 @@ static void RADEONDRITransitionTo3d(ScreenPtr pScreen)
     RADEONChangeSurfaces(pScrn);
     RADEONEnablePageFlip(pScreen);
 
-    if (info->cursor_offset != 0)
+    if (info->cursor)
 	xf86ForceHWCursor (pScreen, TRUE);
 }
 
@@ -2023,7 +2018,7 @@ static void RADEONDRITransitionTo2d(ScreenPtr pScreen)
 
     RADEONChangeSurfaces(pScrn);
 
-    if (info->cursor_offset != 0)
+    if (info->cursor)
 	xf86ForceHWCursor (pScreen, FALSE);
 }
 
@@ -2040,5 +2035,13 @@ void RADEONDRIAllocatePCIGARTTable(ScreenPtr pScreen)
       return;
 
     info->pciGartSize = RADEON_PCIGART_TABLE_SIZE;
+
+    /* allocate space to back up PCIEGART table */
+    info->pciGartBackup = xnfcalloc(1, info->pciGartSize);
+    if (info->pciGartBackup == NULL)
+      return;
+
     info->pciGartOffset = (info->FbMapSize - info->FbSecureSize);
+
+
 }
